"""Media types module providing various media type classes for Litemind.

Import media types directly from their modules to avoid circular imports:
    from litemind.media.types.media_text import Text
    from litemind.media.types.media_image import Image
"""
