"""Media module providing media base classes and types for Litemind.

Import from submodules:
    from litemind.media.media_base import MediaBase
    from litemind.media.media_uri import MediaURI
    from litemind.media.types.media_text import Text
"""
